import latex_scanner
from os.path import join
from django.template import Engine, Context
from django.conf import settings
import pickle


def create_generator(songs, out_dir, dj_engine):
    '''Generate an HTML file with the list of all known songs for the generator.'''
    # Remove songs which share the same .tex file
    files = set()
    remove = set()
    for song in songs:
        if song.latex_file not in files:
            files.add(song.latex_file)
        else:
            remove.add(song.latex_file)
    selected_songs = [s for s in songs if s.latex_file not in remove]
    category_set = set(s.category for s in selected_songs)
    categories = sorted(category_set, key=lambda c:min([s.number for s in songs if s.category == c]))
    dictionary = [(c, [s for s in selected_songs if s.category == c]) for c in categories]


    # Export to html
    context = Context({'categorized_songs': dictionary})
    with open(join(out_dir, "index.html"), 'w') as f:
        f.write(dj_engine.get_template("generador.html").render(context))


def create_argparser():
    '''Parse main's arguments.'''
    parser = latex_scanner.create_argparser()
    parser.add_argument("--generator-dir", required=False, nargs=1, default=["generador_public"],
                        help="The path where the generated files should be placed.")
    return parser


if __name__ == '__main__':
    args = create_argparser().parse_args()
    try:
        with open("songs.dat", "rb") as f:
            songs = pickle.load(f)
    except OSError:
        loader = latex_scanner.SongLoader(args.latex[0])
        if args.other_latex:
            loader.scan_others(args.other_latex[0], args.other_index[0])
        songs = loader.songs
        try:
            with open("songs.dat", "wb") as f:
                pickle.dump(songs, f)
        except OSError:
            pass
    settings.configure(USE_TZ=False, USE_I18N=False)
    e = Engine(dirs=["res_gen/templates/"])
    create_generator(songs, args.generator_dir[0], e)
